//import libraries. 
import de.bezier.data.sql.*;

MySQL msql;

// data containers
ArrayList tArray;
float span = 0; // total number of transactions

// fonts
PFont font0, font1, font2;

// spacing variables
int barWidth = 10;
int barHeight = 20;
int verticalSpacing = 5;
int leftBorder = 10;

String[] sw;

void setup() {
  // canvas setup
  size(900, 900); //size of the sketch window
  colorMode(HSB, 360, 100, 100);
  textAlign(LEFT, CENTER); // (horizontal,vertical)
  translate(width/2, height/2);
  smooth();

  font0 = loadFont("Miso-48.vlw");
  font1 = loadFont("Miso-Bold-48.vlw");
  font2 = loadFont("QuicksandLight-Regular-48.vlw");
  textFont(font0, 17);
  
  // loading stopwords corpus
  String[] lines = loadStrings("stopwords.txt");
  sw = new String[lines.length];
  
  // store stopwords into a var
  for (int i = 0; i < lines.length; i++) {
    sw[i] = lines[i];
  }
  
  // create data containers
  tArray = new ArrayList();

  // connecting to the database
  msql = new MySQL( this, "tango.mat.ucsb.edu", "spl1", "___USER___", "___PASSCODE___" ); //host, database, user, password

  if ( msql.connect() )
  {
    // query
    String query = "SELECT keyword, count(*) as count  FROM `keyword` group by keyword order by count desc limit 100;";
    println(query);
    msql.query(query);
    msql.next();

    int count = 0;
    
    // store fetched rows into tArray
    while ( msql.next () ) {
      tArray.add(new Entity(msql.getString(1), msql.getInt(2)));
      span += msql.getInt(2);
      count++;
//      println(count + "\t" + msql.getString(1) + "\t" + msql.getInt(2));
    }
  }
  else
  {
    println(" connection failed !");
  }

  // compute radius and coordinate of each circle(entity)
  float xSum = 0;
  float rSum = 0;
  for (int i = 0; i < tArray.size(); i++) { 
    // An ArrayList doesn't know what it is storing so we have to CAST the object coming out
    Entity t = (Entity) tArray.get(i);
    
    // calculating radiance of each circle
    t.rad = (float(t.count) / span)*width;
    
    xSum += float(t.count);
    rSum += t.rad;
    t.angle = (xSum/span)*9*PI;
    
    // computing coordinates for the items to visualize
    t.xPosition = cos((xSum/span)*9*PI)*rSum*0.4;
    t.yPosition = sin((xSum/span)*9*PI)*rSum*0.4;
    
    // detect stopword
    for (int j = 0; j < sw.length; j++) {
      if (sw[j].equals(t.keyword.toLowerCase())) {
        t.stopword = true;
        break;
      }
    }
  }
}


void draw() {
  background(#FFFFFF);
  textAlign(CENTER, CENTER);
  noStroke();
  translate(width/2, height/2);
  
  // Title
  textFont(font1, 30);
  fill(#000000);
  text("100 Most Frequent Keywords Appeared in Check-out Transactions in 2012", 0, -410);
  textFont(font0, 15);
  fill(#6C6C6C);
  text("Source: Seattle Public Library", 0, -380);
  text("Byungkyu (Jay) Kang, January, 2013", 0, -360);
  
  // Legend
  textFont(font0, 17);
  text("            :keyword,             :keyword detected as a stopword (e.g. pronoun or preposition)", 0, 400);
  text("Size of Circle: Proportional to the number of occurrence in transactions (2012)", 0, 420);
  fill(#E86946);
  ellipse(-210, 400, 25, 25);
  fill(#87A644);
  ellipse(-120, 400, 25, 25);
  
  float xPrev = 0;
  float yPrev = 0;
  for (int i=0; i<tArray.size(); i++) {
    Entity t = (Entity) tArray.get(i);

    if (i>0) {
      stroke(#959595, 90);
      line(xPrev, yPrev, t.xPosition, t.yPosition);
      noStroke();
    }
    xPrev = t.xPosition;
    yPrev = t.yPosition;
    
    // draw a circle
    if (t.stopword == true) {
      fill(#87A644);
    }else{
      fill(#E86946);
    }
    ellipse(t.xPosition, t.yPosition, t.rad, t.rad);
    if (t.stopword == true) {
      fill(#A0A0A0);
    }else{
      fill(#484848);
    }
    
    // drawing keyword labels
    int fontOffsetX = 0;
    int fontOffsetY = 0;
    if ( abs(t.rad) > 30 ){
        textFont(font1, 20);
        fill(#FFFFFF);
    // lower left
    }else if( t.yPosition > 0 && t.xPosition < 0 ){
      textAlign(RIGHT, TOP);
      fontOffsetX = int(-t.rad/2);
      fontOffsetY = int(t.rad/2);
    // lower right
    }else if( t.yPosition > 0 && t.xPosition > 0 ){
      textAlign(LEFT, TOP);
      fontOffsetX = int(t.rad/2);
      fontOffsetY = int(t.rad/2);
    // upper left
    }else if( t.yPosition < 0 && t.xPosition < 0 ){
      textAlign(RIGHT, BOTTOM);
      fontOffsetX = int(-t.rad/2);
      fontOffsetY = int(-t.rad/2);
    // upper right
    }else if( t.yPosition < 0 && t.xPosition > 0 ){
      textAlign(LEFT, BOTTOM);
      fontOffsetX = int(t.rad/2);
      fontOffsetY = int(-t.rad/2);
    }
    text(t.keyword, t.xPosition + fontOffsetX, t.yPosition + fontOffsetY);
    textFont(font0, 17);
  }
}
